/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package view.dialogs;

import java.awt.Frame;
import java.awt.GridLayout;
import java.util.*;

import javax.swing.*;

public class DialogOpen extends Dialog {

	private String defaultId = "pid";

	private String defaultSex = "sex";

	private String defaultMom = "mom";

	private String defaultDad = "dad";

	private String defaultSexMale = "2";

	private String defaultSexFemale = "1";

	private DefaultComboBoxModel idModel;

	private DefaultComboBoxModel sexModel;

	private DefaultComboBoxModel momModel;

	private DefaultComboBoxModel dadModel;
	
	private JComboBox idComboBox;

	private JComboBox sexComboBox;

	private JComboBox momComboBox;

	private JComboBox dadComboBox;

	private JTextField maleText;

	private JTextField femaleText;
	
	private JCheckBox useQuestionarie, checkErrors;

	public DialogOpen(Frame parent) {
		super(parent, "Import-Settings");
		setSize(350, 300);
	}

	@Override
	protected void createUI(JPanel container) {
		container.setLayout(new GridLayout(8, 2, 10, 10));

		container.add(new JLabel("Id"));
		idComboBox = new JComboBox();
		idModel = new DefaultComboBoxModel();
		idComboBox.setModel(idModel);
		container.add(idComboBox);

		container.add(new JLabel("Sex"));
		sexComboBox = new JComboBox();
		sexModel = new DefaultComboBoxModel();
		sexComboBox.setModel(sexModel);
		container.add(sexComboBox);

		container.add(new JLabel("Mom"));
		momComboBox = new JComboBox();
		momModel = new DefaultComboBoxModel();
		momComboBox.setModel(momModel);
		container.add(momComboBox);

		container.add(new JLabel("Dad"));
		dadComboBox = new JComboBox();
		dadModel = new DefaultComboBoxModel();
		dadComboBox.setModel(dadModel);
		container.add(dadComboBox);

		container.add(new JLabel("Male"));
		maleText = new JTextField();
		container.add(maleText);

		container.add(new JLabel("Female"));
		femaleText = new JTextField();
		container.add(femaleText);
		
		useQuestionarie = new JCheckBox("Import traits from questionarie");
		container.add(useQuestionarie);
		
		checkErrors = new JCheckBox("Check for Errors");
		container.add(checkErrors);		
		
	}

	public void setColData(Vector<String> cols) {
		idModel.removeAllElements();
		sexModel.removeAllElements();
		momModel.removeAllElements();
		dadModel.removeAllElements();

		for (String col : cols) {
			idModel.addElement(col);
			sexModel.addElement(col);
			momModel.addElement(col);
			dadModel.addElement(col);
		}
		if (idModel.getIndexOf(defaultId.toUpperCase()) > -1) {
			idModel.setSelectedItem(defaultId);
		}
		if (idModel.getIndexOf(defaultSex.toUpperCase()) > -1) {
			sexModel.setSelectedItem(defaultSex);
		}
		if (idModel.getIndexOf(defaultMom.toUpperCase()) > -1) {
			momModel.setSelectedItem(defaultMom);
		}
		if (idModel.getIndexOf(defaultDad.toUpperCase()) > -1) {
			dadModel.setSelectedItem(defaultDad);
		}
		maleText.setText(defaultSexMale);
		femaleText.setText(defaultSexFemale);
	}


	public String getId() {
		return (String) idModel.getSelectedItem();
	}

	public String getSex() {
		return (String) sexModel.getSelectedItem();
	}

	public String getMom() {
		return (String) momModel.getSelectedItem();
	}

	public String getDad() {
		return (String) dadModel.getSelectedItem();
	}

	public String getSexFemale() {
		return femaleText.getText();
	}

	public String getSexMale() {
		return maleText.getText();
	}
	
	public boolean useQuestionaire(){
		return useQuestionarie.isSelected();
	}
	
	public void setAllowImportTraits(boolean allow){
		useQuestionarie.setVisible(allow);
	}
	
	public boolean checkErrors(){
		return checkErrors.isSelected();
	}
	
}
