/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package view.tools;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import pedviz.algorithms.filter.Filter;
import core.tools.ToolController;
import java.util.*; 

public class OldFilterView extends ToolView implements ItemListener {

	DefaultComboBoxModel[] traitModel;

	JComboBox[] traitList;

	JComboBox[] mode;

	JComboBox highmode;

	JRadioButton r1;

	JRadioButton r0;

	JLabel labelMode;

	private int n;

	private JComboBox[] minField, maxField;

	private HashMap<String, Vector<String>> values;

	private String[] modes;

	public OldFilterView(ToolController events) {
		super("Filter", events, true);
	}

	protected void createControls(JPanel content) {
		modes = new String[] { "equals", "not equals", "greater", "lesser",
				"between", "not between" };
		n = 2;

		content.setLayout(new GridBagLayout());

		GridBagConstraints constLabel = new GridBagConstraints();
		constLabel.gridx = 0;
		constLabel.gridy = 0;
		constLabel.weightx = 0.5;
		constLabel.weighty = 0;
		constLabel.fill = GridBagConstraints.HORIZONTAL;

		GridBagConstraints constInput = new GridBagConstraints();
		constInput.gridx = 1;
		constInput.gridy = 0;
		constInput.weightx = 0.5;
		constInput.weighty = 0;
		constInput.fill = GridBagConstraints.HORIZONTAL;
		// Mode
		labelMode = new JLabel("Mode");
		content.add(labelMode, constLabel);
		highmode = new JComboBox(new String[] { "Trasparents", "Blinking" });
		content.add(highmode, constInput);

		traitModel = new DefaultComboBoxModel[n];
		traitList = new JComboBox[n];
		mode = new JComboBox[n];
		minField = new JComboBox[n];
		maxField = new JComboBox[n];
		for (int i = 0; i < n; i++) {
			constLabel.gridy = constLabel.gridy + 1;
			constInput.gridy = constInput.gridy + 1;
			traitModel[i] = new DefaultComboBoxModel();
			traitList[i] = new JComboBox();
			traitList[i].setEditable(false);
			traitList[i].setModel(traitModel[i]);
			traitList[i].addItemListener(this);
			content.add(traitList[i], constLabel);
			mode[i] = new JComboBox(modes);
			mode[i].addItemListener(this);
			content.add(mode[i], constInput);
			minField[i] = new JComboBox();
			minField[i] = new JComboBox(new String[] {});
			minField[i].setEditable(true);
			maxField[i] = new JComboBox();
			maxField[i].setEditable(true);
			maxField[i] = new JComboBox(new String[] {});
			constLabel.gridy = constLabel.gridy + 1;
			constInput.gridy = constInput.gridy + 1;
			content.add(minField[i], constLabel);
			content.add(maxField[i], constInput);
			maxField[i].setVisible(false);
			if (i < n - 1) {
				constLabel.gridy = constLabel.gridy + 1;
				constInput.gridy = constInput.gridy + 1;
				ButtonGroup group2 = new ButtonGroup();
				r0 = new JRadioButton("or", true);
				r1 = new JRadioButton("and", false);
				group2.add(r0);
				group2.add(r1);
				content.add(r0, constLabel);
				content.add(r1, constInput);
			}
		}
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		for (int i = 0; i < n; i++) {
			traitList[i].setEnabled(enabled);
			mode[i].setEnabled(enabled);
			minField[i].setEnabled(enabled);
			maxField[i].setEnabled(enabled);
		}
		r1.setEnabled(enabled);
		r0.setEnabled(enabled);
		highmode.setEnabled(enabled);
		labelMode.setEnabled(enabled);
	}

	public void itemStateChanged(ItemEvent e) {
		JComboBox selectedChoice = (JComboBox) e.getSource();
		for (int i = 0; i < n; i++) {
			if (selectedChoice.equals(mode[i])) {
				if (selectedChoice.getSelectedIndex() > 3) {
					maxField[i].setVisible(true);
					updateUI();
				} else {
					maxField[i].setVisible(false);
					updateUI();
				}
			}

			if (selectedChoice.equals(traitList[i])
					&& traitModel[i].getSelectedItem() != null) {
				minField[i].removeAllItems();
				maxField[i].removeAllItems();
				if (values != null) {
					if (values.get(traitModel[i].getSelectedItem().toString()) != null)
						for (Object o : values.get(traitModel[i]
								.getSelectedItem().toString())) {
							minField[i].addItem(o.toString());
							maxField[i].addItem(o.toString());
						}
				}
			}
		}

	}

	public void setTraits(Vector<String> traits) {
		String empty = "-----";
		for (int j = 0; j < n; j++) {
			traitModel[j].removeAllElements();
			traitModel[j].addElement(empty);
			for (String trait: traits) {
				traitModel[j].addElement(trait);
			}
		}
	}

	public void setLookUpTable(HashMap<String, Vector<String>> values) {
		this.values = values;
	}

	public int getSelectedMode() {
		return r0.isSelected() ? Filter.OR : Filter.AND;
	}

	public String getTrait(int index) {
		return traitModel[index].getSelectedItem().toString();
	}

	public int getComperator(int index) {
		return mode[index].getSelectedIndex();
	}

	public String getMinValue(int index) {
		return minField[index].getSelectedItem().toString();
	}

	public String getMaxValue(int index) {
		return maxField[index].getSelectedItem().toString();
	}

	public int getConditions() {
		return n;
	}

	public void setConditions(int n) {
		this.n = n;
	}

	public int getMode() {
		return highmode.getSelectedIndex();
	}
}
